/*
 * Copyright (C) 2014 digitalSTROM.org, Schlieren, Switzerland
 *
 * digitalSTROM ds485-base library is free software: you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation, either version 3 of the License,
 * or (at your option) any later version.
 *
 * digitalSTROM ds485-base library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with digitalSTROM ds485-base library. If not, see
 * <http://www.gnu.org/licenses/>.
 */

#include "ds_packet.h"

void ds485b_pu_from_short(packet_union_t *pkt, const ds485_container_t *cont)
{
    memcpy(&pkt->src, &cont->sourceId, sizeof(pkt->src));
    memcpy(&pkt->dst, &cont->destinationId, sizeof(pkt->dst));
    pkt->transactionId = cont->transactionId;
    pkt->type = v_container;
    pkt->containerType = cont->containerType;
    pkt->u.container = cont;
}

void ds485b_pu_from_long(packet_union_t *pkt, const ds485n_packet_t *lcont)
{
    memcpy(&pkt->src, &lcont->sourceId, sizeof(pkt->src));
    memcpy(&pkt->dst, &lcont->destinationId, sizeof(pkt->dst));
    pkt->transactionId = lcont->transactionId;
    pkt->type = v_packet;
    pkt->containerType = lcont->containerType;
    // lcont is up to 4kByte, careful with copy!
    pkt->u.packet = lcont;
}

void ds485b_pu_sync(packet_union_t *packet)
{
    if (packet->type == v_container) {
        // cast away const
        ds485_container_t *tmp = (ds485_container_t *)packet->u.container;
        memcpy(&tmp->sourceId, &packet->src, sizeof(packet->src));
        memcpy(&tmp->destinationId, &packet->dst, sizeof(packet->dst));
        tmp->transactionId = packet->transactionId;
        tmp->containerType = packet->containerType;
    } else {
        // cast away const
        ds485n_packet_t *tmp = (ds485n_packet_t *)packet->u.packet;
        memcpy(&tmp->sourceId, &packet->src, sizeof(packet->src));
        memcpy(&tmp->destinationId, &packet->dst, sizeof(packet->dst));
        tmp->transactionId = packet->transactionId;
        tmp->containerType = packet->containerType;
    }
}

bool ds485b_pu_is_oversize(const packet_union_t *packet) {
    return (packet->type == v_container) ?
        packet->u.container->length > DS485_MAX_PAYLOAD_SIZE :
        packet->u.packet->length > DS485N_PACKET_MAX_PAYLOAD_SIZE;
}
