/*
 * Copyright (C) 2014 digitalSTROM.org, Schlieren, Switzerland
 *
 * digitalSTROM ds485-base library is free software: you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation, either version 3 of the License,
 * or (at your option) any later version.
 *
 * digitalSTROM ds485-base library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with digitalSTROM ds485-base library. If not, see
 * <http://www.gnu.org/licenses/>.
 */

#include "util.h"

#include <errno.h>
#include <fcntl.h>
#include <limits.h>
#include <stdlib.h>
#include <string.h>
#include <syslog.h>
#include <unistd.h>

#include <digitalSTROM/ds485.h>

#include "log.h"

/* TODO function to drop root privileges */

void panic(const char *why)
{
    report(LOG_EMERG, "%s\n", why);
    abort();
}

int daemonize(void)
{
    int fd;
    pid_t pid;

    switch (pid = fork()) {
    case -1:
        return -1;
    case 0:        /* child side */
        break;
    default:    /* parent side */
        exit(0);
    }

    if (setsid() == -1)
        return -1;
    if (chdir("/") == -1)
        return -1;

    if ((fd = open("/dev/null", O_RDWR, 0)) != -1) {
        dup2(fd, STDIN_FILENO);
        dup2(fd, STDOUT_FILENO);
        dup2(fd, STDERR_FILENO);
        if (fd > 2)
            close(fd);
    }
    return 0;
}

int sscanf_port(const char* str_port)
{
    long val;
    char *endptr;

    errno = 0;    /* To distinguish success/failure after call */
    val = strtol(str_port, &endptr, 10);

    /* Check for various possible errors */
    if ((errno == ERANGE && (val == LONG_MAX || val == LONG_MIN))
        || (errno != 0 && val == 0)) {
        report(LOG_WARNING, "invalid port value %s\n", val);
        return -ERANGE;
    }

    if (endptr == str_port) {
        report(LOG_NOTICE, "No digits were found\n");
        return -EINVAL;
    }

    if (val <= 0 || val > USHRT_MAX) {
        report(LOG_WARNING, "port out of range %s\n", val);
        return -ERANGE;
    }

    return val;
}
